@@ -57,12 +57,11 @@
 #include <openssl/engine.h>
 #include <openssl/x509v3.h>
 
-#define DEBUG_FLAG DEBUG_TLS
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_TLS
 #define DEBUG_HANDSHAKE_LEVEL 5
 #define DEBUG_ASYNC_DETAIL_LEVEL 6
 
-#include "wocky-debug.h"
-#include "wocky-tls-enumtypes.h"
+#include "wocky-debug-internal.h"
 #include "wocky-utils.h"
 
 #include <openssl/ssl.h>
@@ -234,28 +233,6 @@ G_DEFINE_TYPE (WockyTLSOutputStream, wocky_tls_output_stream, G_TYPE_OUTPUT_STRE
                                        WOCKY_TYPE_TLS_OUTPUT_STREAM,         \
                                        WockyTLSOutputStream))
 
-GQuark
-wocky_tls_cert_error_quark (void)
-{
-  static GQuark quark = 0;
-
-  if (quark == 0)
-    quark = g_quark_from_static_string ("wocky-tls-cert-error");
-
-  return quark;
-}
-
-GQuark
-wocky_tls_error_quark (void)
-{
-  static GQuark quark = 0;
-
-  if (quark == 0)
-    quark = g_quark_from_static_string ("wocky-tls-error");
-
-  return quark;
-}
-
 /* Ok: This function tries to retrieve the error that caused a problem from  *
  * bottom of the openssl error stack: The errnum argument is the error code  *
  * returned by the last openssl operation which MAY NOT have come from the   *
@@ -752,29 +729,29 @@ wocky_tls_session_handshake (WockyTLSSession   *session,
 }
 
 /* ************************************************************************* */
-/* adding CA certificates and CRL lists for peer certificate verification    */
-static void
-add_ca_or_crl (WockyTLSSession *session,
-               const gchar *path,
-               const gchar *label)
+/* adding CA certificates lists for peer certificate verification    */
+
+void
+wocky_tls_session_add_ca (WockyTLSSession *session,
+                          const gchar *path)
 {
   gboolean ok = FALSE;
 
   if (!g_file_test (path, G_FILE_TEST_EXISTS))
     {
-      DEBUG ("%s file or path '%s' not accessible", label, path);
+      DEBUG ("CA file or path '%s' not accessible", path);
       return;
     }
 
   if (g_file_test (path, G_FILE_TEST_IS_DIR))
     {
-      DEBUG ("Loading %s directory", label);
+      DEBUG ("Loading CA directory");
       ok = SSL_CTX_load_verify_locations (session->ctx, NULL, path);
     }
 
   if (g_file_test (path, G_FILE_TEST_IS_REGULAR))
     {
-      DEBUG ("Loading %s file", label);
+      DEBUG ("Loading CA file");
       ok = SSL_CTX_load_verify_locations (session->ctx, path, NULL);
     }
 
@@ -783,25 +760,12 @@ add_ca_or_crl (WockyTLSSession *session,
       gulong e, f;
       for (f = e = ERR_get_error (); e != 0; e = ERR_get_error ())
         f = e;
-      DEBUG ("%s '%s' failed: %s", label, path, ERR_error_string (f, NULL));
+      DEBUG ("CA '%s' failed: %s", path, ERR_error_string (f, NULL));
     }
   else
-    DEBUG ("%s '%s' loaded", label, path);
+    DEBUG ("CA '%s' loaded", path);
 }
 
-void
-wocky_tls_session_add_ca (WockyTLSSession *session,
-                          const gchar *path)
-{
-  add_ca_or_crl (session, path, "CA");
-}
-
-void
-wocky_tls_session_add_crl (WockyTLSSession *session,
-                           const gchar *path)
-{
-  add_ca_or_crl (session, path, "CRL");
-}
 /* ************************************************************************* */
 
 void
