@@ -24,8 +24,8 @@
 #include "wocky-tls-handler.h"
 #include "wocky-utils.h"
 
-#define DEBUG_FLAG DEBUG_TLS
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_TLS
+#include "wocky-debug-internal.h"
 
 static void
 real_verify_async (WockyTLSHandler *self,
@@ -50,7 +50,6 @@ struct _WockyTLSHandlerPrivate {
   gboolean ignore_ssl_errors;
 
   GSList *cas;
-  GSList *crls;
 };
 
 static void
@@ -102,12 +101,6 @@ wocky_tls_handler_finalize (GObject *object)
       g_slist_free (self->priv->cas);
     }
 
-  if (self->priv->crls != NULL)
-    {
-      g_slist_foreach (self->priv->crls, (GFunc) g_free, NULL);
-      g_slist_free (self->priv->crls);
-    }
-
   G_OBJECT_CLASS (wocky_tls_handler_parent_class)->finalize (object);
 }
 
@@ -318,30 +311,6 @@ wocky_tls_handler_add_ca (WockyTLSHandler *self,
   return abspath != NULL;
 }
 
-/**
- * wocky_tls_handler_add_crl:
- * @self: a #WockyTLSHandler instance
- * @path: a path to a directory or file containing PEM encoded CRLs
- *
- * This function does not descend subdirectories automatically.
- *
- * Returns: a #gboolean indicating whether the path was resolved.
- * Does not indicate that there was actually a file or directory there
- * or that any CRLs were actually found. The CRLs won't actually be loaded
- * until just before the TLS session setup is attempted.
- */
-gboolean
-wocky_tls_handler_add_crl (WockyTLSHandler *self,
-    const gchar *path)
-{
-  gchar *abspath = wocky_absolutize_path (path);
-
-  if (abspath != NULL)
-    self->priv->crls = g_slist_prepend (self->priv->crls, abspath);
-
-  return abspath != NULL;
-}
-
 GSList *
 wocky_tls_handler_get_cas (WockyTLSHandler *self)
 {
@@ -349,11 +318,3 @@ wocky_tls_handler_get_cas (WockyTLSHandler *self)
 
   return self->priv->cas;
 }
-
-GSList *
-wocky_tls_handler_get_crls (WockyTLSHandler *self)
-{
-  g_assert (WOCKY_IS_TLS_HANDLER (self));
-
-  return self->priv->crls;
-}
