@@ -22,12 +22,17 @@
  * 'g' prefixes changes to 'wocky' and server-side TLS support added.
  *
  */
+#if !defined (WOCKY_H_INSIDE) && !defined (WOCKY_COMPILATION)
+# error "Only <wocky/wocky.h> can be included directly."
+#endif
 
 #ifndef _wocky_tls_h_
 #define _wocky_tls_h_
 
 #include <gio/gio.h>
 
+#include "wocky-tls-enumtypes.h"
+
 #define WOCKY_TYPE_TLS_CONNECTION (wocky_tls_connection_get_type ())
 #define WOCKY_TYPE_TLS_SESSION    (wocky_tls_session_get_type ())
 #define WOCKY_TLS_SESSION(inst)   (G_TYPE_CHECK_INSTANCE_CAST ((inst),   \
@@ -37,8 +42,8 @@
                                    WOCKY_TYPE_TLS_CONNECTION, \
                                    WockyTLSConnection))
 
-typedef struct OPAQUE_TYPE__WockyTLSConnection WockyTLSConnection;
-typedef struct OPAQUE_TYPE__WockyTLSSession WockyTLSSession;
+typedef struct _WockyTLSConnection WockyTLSConnection;
+typedef struct _WockyTLSSession WockyTLSSession;
 
 typedef enum
 {
@@ -82,6 +87,7 @@ GType wocky_tls_session_get_type (void);
 
 int wocky_tls_session_verify_peer (WockyTLSSession    *session,
                                           const gchar        *peername,
+                                          GStrv               extra_identities,
                                           WockyTLSVerificationLevel level,
                                           WockyTLSCertStatus *status);
 GPtrArray *wocky_tls_session_get_peers_certificate (WockyTLSSession *session,
@@ -102,7 +108,6 @@ wocky_tls_session_handshake_finish (WockyTLSSession   *session,
                                     GError       **error);
 
 void wocky_tls_session_add_ca (WockyTLSSession *session, const gchar *path);
-void wocky_tls_session_add_crl (WockyTLSSession *session, const gchar *path);
 
 WockyTLSSession *wocky_tls_session_new (GIOStream *stream);
 
