@@ -1,6 +1,6 @@
 /*
- * gibber-util.c - Code for Wocky utility functions
- * Copyright (C) 2007-2010 Collabora Ltd.
+ * wocky-utils.c - Code for Wocky utility functions
+ * Copyright © 2007–2010 Collabora Ltd.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,10 @@
  *  Copyright (C) 2006-2007 Nokia Corporation
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-utils.h"
 
 #include <string.h>
@@ -133,7 +137,7 @@ wocky_decode_jid (const gchar *jid,
    */
   tmp_resource = strchr (tmp_jid, '/');
 
-  if (tmp_resource)
+  if (tmp_resource != NULL)
     {
       *tmp_resource = '\0';
       tmp_resource++;
@@ -149,7 +153,7 @@ wocky_decode_jid (const gchar *jid,
    */
   tmp_domain = strchr (tmp_jid, '@');
 
-  if (tmp_domain)
+  if (tmp_domain != NULL)
     {
       *tmp_domain = '\0';
       tmp_domain++;
@@ -670,9 +674,11 @@ gchar *
 wocky_absolutize_path (const gchar *path)
 {
   GFile *cwd, *absolute;
-  gchar *ret;
+  gchar *cwd_str, *ret;
 
-  cwd = g_file_new_for_path (g_get_current_dir ());
+  cwd_str = g_get_current_dir ();
+  cwd = g_file_new_for_path (cwd_str);
+  g_free (cwd_str);
 
   if (cwd == NULL)
     return NULL;
