@@ -18,6 +18,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-xmpp-error.h"
 
 #include <stdlib.h>
@@ -25,7 +29,6 @@
 
 #include "wocky-namespaces.h"
 #include "wocky-utils.h"
-#include "wocky-xmpp-error-enumtypes.h"
 
 /* Definitions of XMPP core stanza errors, as per RFC 3920 §9.3; plus the
  * corresponding legacy error codes as described by XEP-0086.
