@@ -22,7 +22,6 @@
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <loudmouth/loudmouth.h>
 #include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_BYTESTREAM
@@ -387,8 +386,8 @@ gabble_bytestream_multiple_accept (GabbleBytestreamIface *iface,
   GabbleBytestreamMultiple *self = GABBLE_BYTESTREAM_MULTIPLE (iface);
   GabbleBytestreamMultiplePrivate *priv =
       GABBLE_BYTESTREAM_MULTIPLE_GET_PRIVATE (self);
-  LmMessage *msg;
-  LmMessageNode *si;
+  WockyStanza *msg;
+  WockyNode *si;
   GList *all_methods;
   gchar *current_method;
 
@@ -413,7 +412,7 @@ gabble_bytestream_multiple_accept (GabbleBytestreamIface *iface,
   g_free (current_method);
   g_list_free (all_methods);
 
-  si = lm_message_node_get_child_with_namespace (
+  si = wocky_node_get_child_ns (
     wocky_stanza_get_top_node (msg), "si", NS_SI);
   g_assert (si != NULL);
 
@@ -431,7 +430,7 @@ gabble_bytestream_multiple_accept (GabbleBytestreamIface *iface,
           GABBLE_BYTESTREAM_STATE_ACCEPTED, NULL);
     }
 
-  lm_message_unref (msg);
+  g_object_unref (msg);
 }
 
 
