@@ -23,12 +23,9 @@
 #include <stdlib.h>
 
 #include <glib-object.h>
-#include <loudmouth/loudmouth.h>
-
+#include <wocky/wocky.h>
 #include <telepathy-glib/base-connection.h>
 
-#include "error.h"
-
 G_BEGIN_DECLS
 
 typedef struct _GabbleBytestreamSocks5 GabbleBytestreamSocks5;
@@ -65,10 +62,10 @@ GType gabble_bytestream_socks5_get_type (void);
                               GabbleBytestreamSocks5Class))
 
 void gabble_bytestream_socks5_add_streamhost (GabbleBytestreamSocks5 *socks5,
-    LmMessageNode *streamhost_node);
+    WockyNode *streamhost_node);
 
 void gabble_bytestream_socks5_connect_to_streamhost (
-    GabbleBytestreamSocks5 *socks5, LmMessage *msg);
+    GabbleBytestreamSocks5 *socks5, WockyStanza *msg);
 
 G_END_DECLS
 
